@@ -1,8 +1,9 @@
-/* Copyright 2000-2004 The Apache Software Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -13,6 +14,11 @@
  * limitations under the License.
  */
 
+/**
+ * @file util_filter.h
+ * @brief Apache filter library
+ */
+
 #ifndef AP_FILTER_H
 #define AP_FILTER_H
 
@@ -29,11 +35,6 @@
 extern "C" {
 #endif
 
-/**
- * @file util_filter.h
- * @brief Apache filter library
- */
-
 /** Returned by the bottom-most filter if no data was written.
  *  @see ap_pass_brigade(). */
 #define AP_NOBODY_WROTE         -1
@@ -44,7 +45,7 @@ extern "C" {
 #define AP_FILTER_ERROR         -3
 
 /**
- * input filtering modes
+ * @brief input filtering modes
  */
 typedef enum {
     /** The filter should return at most readbytes data. */
@@ -71,7 +72,8 @@ typedef enum {
 } ap_input_mode_t;
 
 /**
- * @defgroup filter FILTER CHAIN
+ * @defgroup APACHE_CORE_FILTER Filter Chain
+ * @ingroup  APACHE_CORE
  *
  * Filters operate using a "chaining" mechanism. The filters are chained
  * together into a sequence. When output is generated, it is passed through
@@ -197,25 +199,35 @@ typedef enum {
  */
 
 typedef struct ap_filter_rec_t ap_filter_rec_t;
+typedef struct ap_filter_provider_t ap_filter_provider_t;
 
 /**
- * This structure is used for recording information about the
+ * @brief This structure is used for recording information about the
  * registered filters. It associates a name with the filter's callback
  * and filter type.
  *
  * At the moment, these are simply linked in a chain, so a ->next pointer
  * is available.
+ *
+ * It is used for any filter that can be inserted in the filter chain.
+ * This may be either a httpd-2.0 filter or a mod_filter harness.
+ * In the latter case it contains dispatch, provider and protocol information.
+ * In the former case, the new fields (from dispatch) are ignored.
  */
 struct ap_filter_rec_t {
     /** The registered name for this filter */
     const char *name;
+
     /** The function to call when this filter is invoked. */
     ap_filter_func filter_func;
+
     /** The function to call before the handlers are invoked. Notice
      * that this function is called only for filters participating in
      * the http protocol. Filters for other protocols are to be
-     * initiliazed by the protocols themselves. */
+     * initialized by the protocols themselves.
+     */
     ap_init_filter_func filter_init_func;
+
     /** The type of filter, either AP_FTYPE_CONTENT or AP_FTYPE_CONNECTION.  
      * An AP_FTYPE_CONTENT filter modifies the data based on information 
      * found in the content.  An AP_FTYPE_CONNECTION filter modifies the 
@@ -225,10 +237,21 @@ struct ap_filter_rec_t {
 
     /** The next filter_rec in the list */
     struct ap_filter_rec_t *next;
+
+    /** Providers for this filter */
+    ap_filter_provider_t *providers;
+
+    /** Trace level for this filter */
+    int debug;
+
+    /** Protocol flags for this filter */
+    unsigned int proto_flags;
 };
 
 /**
- * The representation of a filter chain.  Each request has a list
+ * @brief The representation of a filter chain.  
+ *
+ * Each request has a list
  * of these structures which are called in turn to filter the data.  Sub
  * requests get an exact copy of the main requests filter chain.
  */
@@ -304,11 +327,14 @@ AP_DECLARE(ap_filter_rec_t *) ap_register_input_filter(const char *name,
                                           ap_in_filter_func filter_func,
                                           ap_init_filter_func filter_init,
                                           ap_filter_type ftype);
+
 /**
  * This function is used to register an output filter with the system. 
  * After this registration is performed, then a filter may be added 
  * into the filter chain by using ap_add_output_filter() and simply 
- * specifying the name.
+ * specifying the name.  It may also be used as a provider under mod_filter.
+ * This is (equivalent to) ap_register_output_filter_protocol with
+ * proto_flags=0, and is retained for back-compatibility with 2.0 modules.
  *
  * @param name The name to attach to the filter function
  * @param filter_func The filter function to name
@@ -323,6 +349,33 @@ AP_DECLARE(ap_filter_rec_t *) ap_register_output_filter(const char *name,
                                             ap_init_filter_func filter_init,
                                             ap_filter_type ftype);
 
+/* For httpd-2.2 I suggest replacing the above with
+#define ap_register_output_filter(name,ffunc,init,ftype) \
+             ap_register_output_filter_protocol(name,ffunc,init,ftype,0)
+*/
+
+/**
+ * This function is used to register an output filter with the system. 
+ * After this registration is performed, then a filter may be added 
+ * into the filter chain by using ap_add_output_filter() and simply 
+ * specifying the name.  It may also be used as a provider under mod_filter.
+ *
+ * @param name The name to attach to the filter function
+ * @param filter_func The filter function to name
+ * @param filter_init The function to call before the filter handlers 
+ *                    are invoked
+ * @param ftype The type of filter function, either ::AP_FTYPE_CONTENT or
+ *              ::AP_FTYPE_CONNECTION
+ * @param proto_flags Protocol flags: logical OR of AP_FILTER_PROTO_* bits
+ * @see ap_add_output_filter()
+ */
+AP_DECLARE(ap_filter_rec_t *) ap_register_output_filter_protocol(
+                                            const char *name,
+                                            ap_out_filter_func filter_func,
+                                            ap_init_filter_func filter_init,
+                                            ap_filter_type ftype,
+                                            unsigned int proto_flags);
+
 /**
  * Adds a named filter into the filter chain on the specified request record.
  * The filter will be installed with the specified context pointer.
@@ -452,14 +505,14 @@ AP_DECLARE_NONSTD(apr_status_t) ap_filter_flush(apr_bucket_brigade *bb,
 
 /**
  * Flush the current brigade down the filter stack.
- * @param f The current filter
+ * @param f The filter we are passing to
  * @param bb The brigade to flush
  */
 AP_DECLARE(apr_status_t) ap_fflush(ap_filter_t *f, apr_bucket_brigade *bb);
 
 /**
  * Write a buffer for the current filter, buffering if possible.
- * @param f the filter doing the writing
+ * @param f the filter we are writing to
  * @param bb The brigade to buffer into
  * @param data The data to write
  * @param nbyte The number of bytes in the data
@@ -469,7 +522,7 @@ AP_DECLARE(apr_status_t) ap_fflush(ap_filter_t *f, apr_bucket_brigade *bb);
 
 /**
  * Write a buffer for the current filter, buffering if possible.
- * @param f the filter doing the writing
+ * @param f the filter we are writing to
  * @param bb The brigade to buffer into
  * @param str The string to write
  */
@@ -478,7 +531,7 @@ AP_DECLARE(apr_status_t) ap_fflush(ap_filter_t *f, apr_bucket_brigade *bb);
 
 /**
  * Write a character for the current filter, buffering if possible.
- * @param f the filter doing the writing
+ * @param f the filter we are writing to
  * @param bb The brigade to buffer into
  * @param c The character to write
  */
@@ -487,7 +540,7 @@ AP_DECLARE(apr_status_t) ap_fflush(ap_filter_t *f, apr_bucket_brigade *bb);
 
 /**
  * Write an unspecified number of strings to the current filter
- * @param f the filter doing the writing
+ * @param f the filter we are writing to
  * @param bb The brigade to buffer into
  * @param ... The strings to write
  */
@@ -497,7 +550,7 @@ AP_DECLARE_NONSTD(apr_status_t) ap_fputstrs(ap_filter_t *f,
 
 /**
  * Output data to the filter in printf format
- * @param f the filter doing the writing
+ * @param f the filter we are writing to
  * @param bb The brigade to buffer into
  * @param fmt The format string
  * @param ... The argumets to use to fill out the format string
@@ -508,6 +561,32 @@ AP_DECLARE_NONSTD(apr_status_t) ap_fprintf(ap_filter_t *f,
                                            ...)
         __attribute__((format(printf,3,4)));                                    
 
+/**
+ * set protocol requirements for an output content filter
+ * (only works with AP_FTYPE_RESOURCE and AP_FTYPE_CONTENT_SET)
+ * @param f the filter in question
+ * @param proto_flags Logical OR of AP_FILTER_PROTO_* bits
+ */
+AP_DECLARE(void) ap_filter_protocol(ap_filter_t* f, unsigned int proto_flags);
+
+/** Filter changes contents (so invalidating checksums/etc) */
+#define AP_FILTER_PROTO_CHANGE 0x1
+
+/** Filter changes length of contents (so invalidating content-length/etc) */
+#define AP_FILTER_PROTO_CHANGE_LENGTH 0x2
+
+/** Filter requires complete input and can't work on byteranges */
+#define AP_FILTER_PROTO_NO_BYTERANGE 0x4
+
+/** Filter should not run in a proxy */
+#define AP_FILTER_PROTO_NO_PROXY 0x8
+
+/** Filter makes output non-cacheable */
+#define AP_FILTER_PROTO_NO_CACHE 0x10
+
+/** Filter is incompatible with "Cache-Control: no-transform" */
+#define AP_FILTER_PROTO_TRANSFORM 0x20
+
 #ifdef __cplusplus
 }
 #endif
