@@ -67,6 +67,9 @@ static void dumpit(ap_filter_t *f, apr_bucket *b)
             if (nbytes) {
                 obuf = malloc(nbytes+1);    /* use pool? */
                 memcpy(obuf, buf, nbytes);
+#if APR_CHARSET_EBCDIC
+                ap_xlate_proto_from_ascii(obuf, nbytes);
+#endif
                 obuf[nbytes] = '\0';
                 ap_log_error(APLOG_MARK, ptr->loglevel, 0, c->base_server,
                      "mod_dumpio:  %s (%s-%s): %s",
